# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'easy_type/helpers'
require 'puppet_x/vulnerability/selection'
require 'puppet_x/vulnerability/config'

# top level class
module Puppet
  # The vulnerability_status custom type
  Type.newtype(:vulnerability_status) do
    include EasyType
    include EasyType::Helpers
    extend ::Vulnerability::Selection
    extend ::Vulnerability::Config

    desc <<-DESC
    Manage the status of the vulnerabilities on your system.

    Using this type, you can start to manage the number of vulnerabilities you allow on your system.
    Here is an example on how to use it:

          vulnerability_status { 'master.example.com':
            critical   => 2,
            high       => 90,
            allow_list => ['CVE-2018-11749'],
            when_more  => error,
          }

    In this example, you allow a maximum of 2 critical and 90 high vulnerabilities on your system. You#{' '}
    know that vulnerability CVE-2018-11749 is on your system and for now, you don't want it to allow this
    CVE and not include it in the count.
    DESC

    on_prefetch do |resources, provider|
      node_name = Facter.value('fqdn')
      current_resource = resources[node_name]
      return if current_resource.nil?

      # Fetch the CVE's if not (yet) available assume none
      cve_list = Facter.value(:cve_list) || []
      cve_list = filter_cves(cve_list, current_resource.allow_list)
      properties = { :name => node_name }
      selected_levels(config_data[:level]).each do |level|
        properties[level.downcase.to_sym] = cve_list.count { |_k, v| v['severity'] == level }
      end
      current_resource.provider = provider.new(properties)
      resources
    end

    on_create do
      nil
    end

    on_modify do
      # Allow individual properties to do their stuff
    end

    on_destroy do
      nil
    end

    parameter :name
    parameter :when_more
    parameter :allow_list
    property  :critical
    property  :high
    property  :medium
    property  :low
    property  :negligible
    property  :unknown
  end
end
