#
# vulnerability::vulnerability
#
# @summary The top-level class of the vulnerability module.
# This class ensures that the correct version of [`grype`](https://github.com/anchore/grype) is installed and configured and that on the specified interval your systems are scanned for vulnerabilities.
#
# @param update
#    Update the vulnerability database on every Puppet run.
#    When you set this value to `true`, Puppet will check the vulnerability database on every run and update it when it detects a new version.
#    Although setting it to true is the best setting security-wise, it can introduce dynamic changes to your Puppet run's that you don't want. When you want more controlled updates, set this value to false and make sure that the `vulnerability::update` class is scheduled in some other way.
#    Even when you set this value to `false`, Puppet will do an update on the initial run where [`grype`](https://github.com/anchore/grype) is installed. This is required to at least have an initial vulnerability database.
#
# @param guard
#    When you set this value to `true`, Puppet start's to guard the number of vulnerabilities on your system.
#    Check the [`::vulnerability::guard` class](./guard.html) for details.
#    The default value is `false` meaning no automatic checks on the vulnerability status.
#
# @param remediate
#    When you set this value to `true`, Puppet will remediate the specfied CVE's.
#    See the [remediate class](../remediate) for details.
#    The default value is `true` meaning no automatic checks on the vulnerability status.
#
#
# See the file "LICENSE" for the full license governing this code.
#
class vulnerability (
  Boolean $guard,
  Boolean $remediate,
  Boolean $update
) {
  include stdlib
  contain vulnerability::install
  contain vulnerability::setup

  Class['vulnerability::install']
  -> Class['vulnerability::setup']

  if $update or $facts['grype_version'] == 'not-installed' {
    contain vulnerability::update

    Class['vulnerability::setup']
    -> Class['vulnerability::update']
  }
  if $guard {
    contain vulnerability::guard

    Class['vulnerability::update']
    -> Class['vulnerability::guard']
  }

  if $remediate {
    contain vulnerability::remediate

    Class['vulnerability::setup']
    -> Class['vulnerability::remediate']
  }
}
